#include "include/config.h"
#include <iostream>
#include <fstream>
#include <sstream>
#include <algorithm>
#include <cctype>

namespace openmanus {

Config::Config(const std::string& config_file) {
    load(config_file);
}

bool Config::load(const std::string& config_file) {
    std::ifstream file(config_file);
    if (!file.is_open()) {
        std::cerr << "无法打开配置文件: " << config_file << std::endl;
        return false;
    }

    std::string line;
    std::string current_section;

    while (std::getline(file, line)) {
        // 去除前后空白
        line.erase(0, line.find_first_not_of(" \t"));
        line.erase(line.find_last_not_of(" \t") + 1);

        // 跳过空行和注释
        if (line.empty() || line[0] == '#') {
            continue;
        }

        // 处理节
        if (line[0] == '[' && line[line.length() - 1] == ']') {
            current_section = line.substr(1, line.length() - 2);
            continue;
        }

        // 处理键值对
        size_t pos = line.find('=');
        if (pos != std::string::npos) {
            std::string key = line.substr(0, pos);
            std::string value = line.substr(pos + 1);

            // 去除键和值的前后空白
            key.erase(0, key.find_first_not_of(" \t"));
            key.erase(key.find_last_not_of(" \t") + 1);
            value.erase(0, value.find_first_not_of(" \t"));
            value.erase(value.find_last_not_of(" \t") + 1);

            // 处理引号
            if (value.length() >= 2 && (value[0] == '"' && value[value.length() - 1] == '"' ||
                                        value[0] == '\'' && value[value.length() - 1] == '\'')) {
                value = value.substr(1, value.length() - 2);
            }

            config_data_[current_section][key] = value;
        }
    }

    file.close();
    return true;
}

std::string Config::getString(const std::string& section, const std::string& key, const std::string& default_value) const {
    auto section_it = config_data_.find(section);
    if (section_it != config_data_.end()) {
        auto key_it = section_it->second.find(key);
        if (key_it != section_it->second.end()) {
            return key_it->second;
        }
    }
    return default_value;
}

int Config::getInt(const std::string& section, const std::string& key, int default_value) const {
    std::string value = getString(section, key, "");
    if (!value.empty()) {
        try {
            return std::stoi(value);
        } catch (...) {
            // 转换失败，返回默认值
        }
    }
    return default_value;
}

double Config::getDouble(const std::string& section, const std::string& key, double default_value) const {
    std::string value = getString(section, key, "");
    if (!value.empty()) {
        try {
            return std::stod(value);
        } catch (...) {
            // 转换失败，返回默认值
        }
    }
    return default_value;
}

bool Config::getBool(const std::string& section, const std::string& key, bool default_value) const {
    std::string value = getString(section, key, "");
    if (!value.empty()) {
        // 转换为小写
        std::transform(value.begin(), value.end(), value.begin(),
                       [](unsigned char c) { return std::tolower(c); });

        if (value == "true" || value == "yes" || value == "1") {
            return true;
        } else if (value == "false" || value == "no" || value == "0") {
            return false;
        }
    }
    return default_value;
}

} // namespace openmanus 