#include "include/flow/planning_flow.h"
#include <iostream>
#include <chrono>
#include <sstream>
#include <iomanip>
#include "mcp/include/mcp_message.h"

namespace openmanus {

PlanningFlow::PlanningFlow(std::shared_ptr<AgentBase> agent)
    : BaseFlow(agent) {
    // 生成计划ID
    auto now = std::chrono::system_clock::now();
    auto now_time_t = std::chrono::system_clock::to_time_t(now);
    std::stringstream ss;
    ss << "plan_" << std::put_time(std::localtime(&now_time_t), "%Y%m%d%H%M%S");
    active_plan_id_ = ss.str();
}

PlanningFlow::PlanningFlow(const std::vector<std::shared_ptr<AgentBase>>& agents)
    : BaseFlow(agents) {
    // 生成计划ID
    auto now = std::chrono::system_clock::now();
    auto now_time_t = std::chrono::system_clock::to_time_t(now);
    std::stringstream ss;
    ss << "plan_" << std::put_time(std::localtime(&now_time_t), "%Y%m%d%H%M%S");
    active_plan_id_ = ss.str();
}

std::string PlanningFlow::execute(const std::string& input_text) {
    auto primary_agent = getPrimaryAgent();
    if (!primary_agent) {
        throw std::runtime_error("没有可用的代理");
    }
    
    std::cout << "使用规划流程执行任务..." << std::endl;
    
    // 构建提示，包含计划状态
    std::string prompt = "CURRENT PLAN STATUS:\n" + getPlanStatus() + "\n\n" + input_text;
    
    // 执行主要代理
    std::string result = primary_agent->run(prompt);
    
    // 获取当前步骤索引
    int current_step_index = getCurrentStepIndex();
    
    // 如果有工具调用，更新步骤状态
    // 注意：这里假设ToolCallAgent会在执行工具后更新消息历史
    // 在实际实现中，需要根据具体情况调整
    
    return result;
}

std::string PlanningFlow::getPlanStatus() {
    auto primary_agent = getPrimaryAgent();
    if (!primary_agent) {
        return "没有可用的代理";
    }
    
    // 构建获取计划状态的参数
    mcp::json params = {
        {"action", "get_plan_status"},
        {"plan_id", active_plan_id_}
    };
    
    // 创建工具调用
    mcp::json tool_call = {
        {"name", "planning"},
        {"params", params}
    };
    
    try {
        // 执行工具调用
        // 注意：这里假设ToolBase有一个静态方法可以执行工具调用
        // 在实际实现中，需要根据具体情况调整
        mcp::json result = primary_agent->executeToolCall("planning", params);
        
        // 解析结果
        if (result.contains("success") && result["success"].get<bool>()) {
            if (result.contains("plan")) {
                return result["plan"].dump(4);
            }
        }
        
        // 如果没有计划，创建一个新计划
        if (!result.contains("success") || !result["success"].get<bool>()) {
            mcp::json create_params = {
                {"action", "create_plan"},
                {"plan_id", active_plan_id_},
                {"title", "任务计划"},
                {"description", "执行用户请求的任务"}
            };
            
            mcp::json create_result = primary_agent->executeToolCall("planning", create_params);
            
            if (create_result.contains("success") && create_result["success"].get<bool>()) {
                active_plan_id_ = create_result["plan_id"];
                
                // 重新获取计划状态
                return getPlanStatus();
            }
        }
        
        return "无法获取计划状态";
    } catch (const std::exception& e) {
        return std::string("获取计划状态出错: ") + e.what();
    }
}

bool PlanningFlow::updateStepStatus(const std::string& step_id, const std::string& status) {
    auto primary_agent = getPrimaryAgent();
    if (!primary_agent) {
        return false;
    }
    
    // 构建更新步骤状态的参数
    mcp::json params = {
        {"action", "update_step_status"},
        {"plan_id", active_plan_id_},
        {"step_id", step_id},
        {"status", status}
    };
    
    try {
        // 执行工具调用
        mcp::json result = primary_agent->executeToolCall("planning", params);
        
        // 解析结果
        if (result.contains("success") && result["success"].get<bool>()) {
            return true;
        }
        
        return false;
    } catch (const std::exception& e) {
        std::cerr << "更新步骤状态出错: " << e.what() << std::endl;
        return false;
    }
}

int PlanningFlow::getCurrentStepIndex() {
    auto primary_agent = getPrimaryAgent();
    if (!primary_agent) {
        return -1;
    }
    
    // 构建获取计划状态的参数
    mcp::json params = {
        {"action", "get_plan_status"},
        {"plan_id", active_plan_id_}
    };
    
    try {
        // 执行工具调用
        mcp::json result = primary_agent->executeToolCall("planning", params);
        
        // 解析结果
        if (result.contains("success") && result["success"].get<bool>()) {
            if (result.contains("plan") && result["plan"].contains("current_step_index")) {
                return result["plan"]["current_step_index"];
            }
        }
        
        return -1;
    } catch (const std::exception& e) {
        std::cerr << "获取当前步骤索引出错: " << e.what() << std::endl;
        return -1;
    }
}

} // namespace openmanus 